package com.nationrel.modules.fta.entity;

import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.annotation.Version;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.Max;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;


/**
 * @TableName fta_analysis_nodes
 */
@Schema(description = "节点信息表")
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("fta_node")
public class FtaNode {

    @Schema(description = "分析ID")
    private String analysisId;

    @Schema(description = "事件ID")
    @TableId
    private String id;

    @Schema(description = "父节点ID")
    private String fatherId;

    @Schema(description = "转移门对应的子分析id")
    private String childrenAnalysisId;

    @Schema(description = "事件编号")
    private String code;

    @Schema(description = "事件名称")
    private String name;

    @Schema(description = "1：顶事件 2：中间事件 3：底事件 4：未开发事件 5：重复事件 6：条件事件 7：转移门")
    private String typeCode;

    @Schema(description = "重复事件的对应事件id")
    private String masterId;

    @Schema(description = "分布类型编号")
    private String distributionTypeCode;

    @Schema(description = "p1Value")
    @Max(value = 1000000, message = "checkData")
    private Double p1Value;

    @Schema(description = "p2Value")
    @Max(value = 1000000, message = "checkData")
    private Double p2Value;

    @Schema(description = "p3Value")
    @Max(value = 1000000, message = "checkData")
    private Double p3Value;

    @Schema(description = "Q(固定失效概率)")
    private String q;

    @Schema(description = "割集")
    private String cutset;

    @Schema(description = "计算标识")
    private Integer calFlag;

    @Schema(description = "KN节点的必要路径")
    private Integer requiredPath;

    @Schema(description = "节点折叠/展开")
    private Boolean leaf;

    @Schema(description = "节点折叠/展开")
    private Boolean disableMove;

    @Schema(description = "节点层级")
    private Integer nodeLevel;

    @Schema(description = "控制节点顺序（前端排序功能）")
    private Integer edgeOrder;

    @Version
    private Integer versionNo;

    @TableLogic
    @Schema(description = "删除状态")
    private String delFlag;
}
