package com.nationrel.modules.fta.general.common;


import com.fasterxml.jackson.annotation.JsonIgnore;
import com.nationrel.modules.fta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.fta.general.domain.enums.reliaqube.ReliaqubeProgramExceptionEnum;
import com.nationrel.modules.fta.general.domain.enums.weibull.EwbCustomPromptEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;

/**
 * 创建人是: hcs 创建时间: 2018/2/7 0007.
 * <p>
 * 保证序列化json的时候,如果是null的对象,key也会消失
 */
@AllArgsConstructor
@NoArgsConstructor
@Builder
@Data
public class ServerResponse<T> implements Serializable {

    private int status;
    private String message;
    private T data;

    private ServerResponse(int status) {
        this.status = status;
    }

    private ServerResponse(int status, T data) {
        this.status = status;
        this.data = data;
    }

    private ServerResponse(int status, String message) {
        this.status = status;
        this.message = message;
    }

    @JsonIgnore
    //使之不在json序列化结果当中
    public boolean isSuccess() {
        return this.status == ResponseStatus.SUCCESS.getStatus();
    }

    public int getStatus() {
        return status;
    }

    public T getData() {
        return data;
    }

    public String getMessage() {
        return message;
    }


    public static <T> ServerResponse<T> createBySuccess() {
        return new ServerResponse<T>(ResponseStatus.SUCCESS.getStatus());
    }

    public static <T> ServerResponse<T> createBySuccess(T data) {
        return new ServerResponse<T>(ResponseStatus.SUCCESS.getStatus(), data);
    }

    public static <T> ServerResponse<T> createBySuccess(Integer status, T data) {
        return new ServerResponse<T>(status, data);
    }

    public static <T> ServerResponse<T> createBySuccess(String msg, T data) {
        return new ServerResponse<T>(ResponseStatus.SUCCESS.getStatus(), msg, data);
    }


    public static <T> ServerResponse<T> createByError() {
        return new ServerResponse<T>(ResponseStatus.ERROR.getStatus(), ResponseStatus.ERROR.getMessage());
    }


    public static <T> ServerResponse<T> createByErrorMessage(String errorMessage) {
        return new ServerResponse<T>(ResponseStatus.ERROR.getStatus(), errorMessage);
    }

    public static <T> ServerResponse<T> createByErrorCodeMessage(int errorCode, String errorMessage) {
        return new ServerResponse<T>(errorCode, errorMessage);
    }

    public static <T> ServerResponse<T> createByErrorMessage(EwbCustomPromptEnum enumStr, Integer languageKey) {
        return new ServerResponse<T>(ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex(), EwbCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey));
    }

    public static <T> ServerResponse<T> createByErrorMessage(ReliaqubeParamDataCheckEnum enumStr, Integer languageKey) {
        return new ServerResponse<T>(ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex(), ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(enumStr, languageKey));
    }
}
