package com.nationrel.modules.fta.general.domain.enums.fta;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.nationrel.modules.fta.general.util.common.BasicDataUtil;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Aimee on 2020/10/22.
 */
public enum FtaNodeEnum {
    /**
     * 1：顶事件 2：中间事件 3：底事件 4：未开发事件 5：重复事件 6：转移门 7：条件事件
     */
    /**
     * 11：与门 12：或门 13：表决门 14：非门 15：禁门 16：NOR 17：NAND
     */
    顶事件("1", "顶事件", "", "", ""),
    中间事件("2", "中间事件", "", "", ""),
    底事件("3", "底事件", "circle", "", ""),
    未展开事件("4", "未展开事件", "", "", ""),
    重复事件("5", "重复事件", "mirror", "", ""),
    候补节点("14", "候补节点", "", "", ""),
    转移门("6", "转移门", "transfer", "", ""),


    与门("11", "与门", "AND", "", "*"),
    或门("12", "或门", "OR", "", "+"),
    // 表决门 特定情况时当做与门计算 mathematics *
    表决门("13", "表决门", "KOON", "", "*");

    private String index;

    private String name;

    // 计算接口需要的参数值
    private String uriName;

    private String figure;

    private String mathematics;

    FtaNodeEnum(String index, String name, String uriName, String figure, String mathematics) {
        this.index = index;
        this.name = name;
        this.uriName = uriName;
        this.figure = figure;
        this.mathematics = mathematics;
    }

    public String getIndex() {
        return index;
    }

    public void setIndex(String index) {
        this.index = index;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getUriName() {
        return uriName;
    }

    public void setUriName(String uriName) {
        this.uriName = uriName;
    }

    public String getFigure() {
        return figure;
    }

    public void setFigure(String figure) {
        this.figure = figure;
    }

    public String getMathematics() {
        return mathematics;
    }

    public void setMathematics(String mathematics) {
        this.mathematics = mathematics;
    }

    // Implementing a fromString method on an enum type
    private static final Map<String, FtaNodeEnum> JSON_ARRAY_MAP = new HashMap<>();

    public static final JSONArray JSON_ARRAY = new JSONArray();

    static {
        // Initialize map from constant name to enum constant
        for (FtaNodeEnum enumForEach : values()) {
            JSON_ARRAY_MAP.put(enumForEach.getIndex(), enumForEach);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("value", enumForEach.toString());
            jsonObject.put("key", enumForEach.getIndex());
            JSON_ARRAY.add(jsonObject);
        }
    }

    public static String getFigureByIndex(String index) {
        for (FtaNodeEnum e : values()) {
            if (e.getIndex().equals(index)) {
                return e.getFigure();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getIndexByFigure(String figure) {
        for (FtaNodeEnum e : values()) {
            if (e.getFigure().equals(figure)) {
                return e.getIndex();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getNameByFigure(String figure) {
        for (FtaNodeEnum e : values()) {
            if (e.getFigure().equals(figure)) {
                return e.getName();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getUriNameByIndex(String index) {
        for (FtaNodeEnum e : values()) {
            if (e.getIndex().equals(index)) {
                return e.getUriName();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getNameByIndex(String index) {
        for (FtaNodeEnum e : values()) {
            if (e.getIndex().equals(index)) {
                return e.getName();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getMathematicsByUriName(String uriName) {
        for (FtaNodeEnum e : values()) {
            if (e.getUriName().equals(uriName)) {
                return e.getMathematics();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

}
