package com.nationrel.modules.fta.general.domain.enums.reliaqube;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.nationrel.modules.fta.general.util.common.BasicDataUtil;

import java.util.HashMap;
import java.util.Map;

/**
 * 语言枚举类
 * @author hcs on 2020/7/15.
 */
public enum ReliaqubeLanguageKeyEnum {
    /**
     * 语言枚举类
     */
    中文(1, "zh", "Chinese", "中文"),

    英语(2, "en", "English","英语"),

    俄语(3, "ru", "Russian","俄语"),

    法语(4, "fra", "French","法语"),

    德语(5, "de", "Deutsch","德语"),

    日语(6, "jp", "Japanese","日语"),

    丹麦语(7, "dan", "Danish","丹麦语"),

    保加利亚语(8, "bul", "Bulgarian","保加利亚语"),

    匈牙利语(9, "hu", "Hungarian","匈牙利语"),

    希腊语(10, "el", "Greek","希腊语"),

    意大利语(11, "it", "Italian","意大利语"),

    捷克语(12, "cs", "Czech","捷克语"),

    斯洛文尼亚语(13, "slo", "Slovenian","斯洛文尼亚语"),

    韩语(14, "kor", "Korean","韩语"),

    波兰语(15, "pl", "Polish","波兰语"),

    泰语(16, "th", "Thai","泰语"),

    爱沙尼亚语(17, "est", "Estonian","爱沙尼亚语"),

    瑞典语(18, "swe", "Swedish","瑞典语"),

    罗马尼亚语(19, "rom", "Romanian","罗马尼亚语"),

    芬兰语(20, "fin", "Finnish","芬兰语"),

    荷兰语(21, "nl", "Dutch","荷兰语"),

    葡萄牙语(22, "pt", "Portuguese","葡萄牙语"),

    西班牙语(23, "spa", "Spanish","西班牙语"),

    越南语(24, "vie", "Vietnamese","越南语"),

    阿拉伯语(25, "ara", "Arabic","阿拉伯语"),

    繁体中文(26, "cht", "ChineseTW","繁体中文");

    private Integer index;
    private String code;
    private String name;
    private String value;

    ReliaqubeLanguageKeyEnum(Integer index, String code, String name, String value) {
        this.index = index;
        this.code = code;
        this.name = name;
        this.value = value;
    }

    public Integer getIndex() {
        return index;
    }

    public void setIndex(Integer index) {
        this.index = index;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }
    // Implementing a fromString method on an enum type
    public static final Map<Integer, ReliaqubeLanguageKeyEnum> JSON_ARRAY_MAP = new HashMap<>();

    public static final JSONArray JSON_ARRAY = new JSONArray();

    static {
        // Initialize map from constant name to enum constant
        for (ReliaqubeLanguageKeyEnum languageKeyEnum : values()) {
            JSON_ARRAY_MAP.put(languageKeyEnum.getIndex(), languageKeyEnum);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("value", languageKeyEnum.getName());
            jsonObject.put("descOne", languageKeyEnum.getValue());
            jsonObject.put("key", languageKeyEnum.getIndex());
            JSON_ARRAY.add(jsonObject);
        }
    }

    public static boolean checkLanguage(String language) {
        ReliaqubeLanguageKeyEnum[] languageEnums = ReliaqubeLanguageKeyEnum.values();
        for (ReliaqubeLanguageKeyEnum languageEnum : languageEnums) {
            if (languageEnum.code.equals(language)) {
                return true;
            }
        }
        return false;
    }

    public static String getLanguageCodeByIndex(int index){
        ReliaqubeLanguageKeyEnum[] languageEnums = ReliaqubeLanguageKeyEnum.values();
        for (ReliaqubeLanguageKeyEnum languageEnum : languageEnums) {
            if (languageEnum.index.equals(index)) {
                return languageEnum.code;
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getLanguageValueByIndex(int index){
        ReliaqubeLanguageKeyEnum[] languageEnums = ReliaqubeLanguageKeyEnum.values();
        for (ReliaqubeLanguageKeyEnum languageEnum : languageEnums) {
            if (languageEnum.index.equals(index)) {
                return languageEnum.value;
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getLanguageNameByIndex(int index){
        ReliaqubeLanguageKeyEnum[] languageEnums = ReliaqubeLanguageKeyEnum.values();
        for (ReliaqubeLanguageKeyEnum languageEnum : languageEnums) {
            if (languageEnum.index.equals(index)) {
                return languageEnum.name;
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static ReliaqubeLanguageKeyEnum getLanguageEnumByCode(String language) {
        ReliaqubeLanguageKeyEnum[] languageEnums = ReliaqubeLanguageKeyEnum.values();
        for (ReliaqubeLanguageKeyEnum languageEnum : languageEnums) {
            if (languageEnum.code.equals(language)) {
                return languageEnum;
            }
        }
        return null;
    }



    /**
     * IQ-FMEA当前使用语言对应
     */
    public static final Map<String, Integer> IQ_USE_LANG_MAP = new HashMap<String, Integer>();
    static
    {
        IQ_USE_LANG_MAP.put("Chinese", ReliaqubeLanguageKeyEnum.中文.getIndex());
        IQ_USE_LANG_MAP.put("English", ReliaqubeLanguageKeyEnum.英语.getIndex());
        IQ_USE_LANG_MAP.put("Russian", ReliaqubeLanguageKeyEnum.俄语.getIndex());
        IQ_USE_LANG_MAP.put("French", ReliaqubeLanguageKeyEnum.法语.getIndex());
        IQ_USE_LANG_MAP.put("Deutsch", ReliaqubeLanguageKeyEnum.德语.getIndex());
        IQ_USE_LANG_MAP.put("Japanese", ReliaqubeLanguageKeyEnum.日语.getIndex());
        IQ_USE_LANG_MAP.put("Danish", ReliaqubeLanguageKeyEnum.丹麦语.getIndex());
        IQ_USE_LANG_MAP.put("Hungarian", ReliaqubeLanguageKeyEnum.匈牙利语.getIndex());
        IQ_USE_LANG_MAP.put("Greek", ReliaqubeLanguageKeyEnum.希腊语.getIndex());
        IQ_USE_LANG_MAP.put("Italian", ReliaqubeLanguageKeyEnum.意大利语.getIndex());
        IQ_USE_LANG_MAP.put("Czech", ReliaqubeLanguageKeyEnum.捷克语.getIndex());
        IQ_USE_LANG_MAP.put("Slovenian", ReliaqubeLanguageKeyEnum.斯洛文尼亚语.getIndex());
        IQ_USE_LANG_MAP.put("Korean", ReliaqubeLanguageKeyEnum.韩语.getIndex());
        IQ_USE_LANG_MAP.put("Polish", ReliaqubeLanguageKeyEnum.波兰语.getIndex());
        IQ_USE_LANG_MAP.put("Swedish", ReliaqubeLanguageKeyEnum.瑞典语.getIndex());
        IQ_USE_LANG_MAP.put("Romanian", ReliaqubeLanguageKeyEnum.罗马尼亚语.getIndex());
        IQ_USE_LANG_MAP.put("Finnish", ReliaqubeLanguageKeyEnum.芬兰语.getIndex());
        IQ_USE_LANG_MAP.put("Dutch", ReliaqubeLanguageKeyEnum.荷兰语.getIndex());
        IQ_USE_LANG_MAP.put("Portuguese", ReliaqubeLanguageKeyEnum.葡萄牙语.getIndex());
        IQ_USE_LANG_MAP.put("Spanish", ReliaqubeLanguageKeyEnum.西班牙语.getIndex());
        IQ_USE_LANG_MAP.put("Vietnamese", ReliaqubeLanguageKeyEnum.越南语.getIndex());
        IQ_USE_LANG_MAP.put("ChineseTW", ReliaqubeLanguageKeyEnum.繁体中文.getIndex());
    }

    /**
     * IQ-FMEA多语对应对应
     */
    public static final Map<String, Integer> IQ_LANG_MAP = new HashMap<String, Integer>();
    static
    {
        IQ_LANG_MAP.put("ZH", ReliaqubeLanguageKeyEnum.中文.getIndex());
        IQ_LANG_MAP.put("EN", ReliaqubeLanguageKeyEnum.英语.getIndex());
        IQ_LANG_MAP.put("RU", ReliaqubeLanguageKeyEnum.俄语.getIndex());
        IQ_LANG_MAP.put("FR", ReliaqubeLanguageKeyEnum.法语.getIndex());
        IQ_LANG_MAP.put("DE", ReliaqubeLanguageKeyEnum.德语.getIndex());
        IQ_LANG_MAP.put("JA", ReliaqubeLanguageKeyEnum.日语.getIndex());
        IQ_LANG_MAP.put("DA", ReliaqubeLanguageKeyEnum.丹麦语.getIndex());
        IQ_LANG_MAP.put("HU", ReliaqubeLanguageKeyEnum.匈牙利语.getIndex());
        IQ_LANG_MAP.put("EL", ReliaqubeLanguageKeyEnum.希腊语.getIndex());
        IQ_LANG_MAP.put("IT", ReliaqubeLanguageKeyEnum.意大利语.getIndex());
        IQ_LANG_MAP.put("CS", ReliaqubeLanguageKeyEnum.捷克语.getIndex());
        IQ_LANG_MAP.put("SL", ReliaqubeLanguageKeyEnum.斯洛文尼亚语.getIndex());
        IQ_LANG_MAP.put("PL", ReliaqubeLanguageKeyEnum.波兰语.getIndex());
        IQ_LANG_MAP.put("SV", ReliaqubeLanguageKeyEnum.瑞典语.getIndex());
        IQ_LANG_MAP.put("RO", ReliaqubeLanguageKeyEnum.罗马尼亚语.getIndex());
        IQ_LANG_MAP.put("FI", ReliaqubeLanguageKeyEnum.芬兰语.getIndex());
        IQ_LANG_MAP.put("NL", ReliaqubeLanguageKeyEnum.荷兰语.getIndex());
        IQ_LANG_MAP.put("PT", ReliaqubeLanguageKeyEnum.葡萄牙语.getIndex());
        IQ_LANG_MAP.put("ES", ReliaqubeLanguageKeyEnum.西班牙语.getIndex());
        IQ_LANG_MAP.put("Vietnamese", ReliaqubeLanguageKeyEnum.越南语.getIndex());
    }
}