package com.nationrel.modules.fta.general.util.common;

import javax.crypto.Cipher;
import java.io.*;
import java.lang.reflect.Array;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static java.util.regex.Pattern.compile;

/**
 * @version V1.0
 * @ClassName StringUtil
 * @Description 字符串处理类
 */
public class StringUtil {

    /**
     * <p>Title: </p>
     * <p>Description: </p>
     */
    private StringUtil() {
        super();
    }

    /**
     * @param str
     * @return String
     * @Title: toCamelCase
     * @Description: 转换成驼峰命名
     */
    public static String toCamelCase(String str) {
        str = str.toLowerCase();
        StringBuilder sb = new StringBuilder();
        boolean upperCase = false;
        for (int i = 0; i < str.length(); i++) {
            char ch = str.charAt(i);
            if (ch == '_') {
                upperCase = true;
            } else {
                sb.append(upperCase ? Character.toUpperCase(ch) : ch);
                upperCase = false;
            }
        }
        return sb.toString();
    }

    /**
     * @param str
     * @param length
     * @param c
     * @return String
     * @Title: rightPad
     * @Description: 补充串（左侧）
     */
    public static String leftPad(String str, int length, char c) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < length - str.length(); i++) {
            sb.append(c);
        }
        sb.append(str);
        return sb.toString();
    }

    /**
     * @param str
     * @param length
     * @param c
     * @return String
     * @Title: rightPad
     * @Description: 补充串（右侧）
     */
    public static String rightPad(String str, int length, char c) {
        StringBuilder sb = new StringBuilder();
        sb.append(str);
        for (int i = 0; i < length - str.length(); i++) {
            sb.append(c);
        }
        return sb.toString();
    }

    /**
     * @param fileName
     * @return String
     * @Title: fileExtensionOf
     * @Description: 文件扩展名
     */
    public static String fileExtensionOf(String fileName) {
        return fileName.substring(fileName.lastIndexOf('.') + 1);
    }

    /**
     * @param obj
     * @return String
     * @Title:
     * @Description: 对象转化为字符串
     */
    public static String parseString(Object obj) {
        return parseString(obj, "");
    }

    /**
     * @param obj
     * @return String
     * @Title:
     * @Description: 对象转化为字符串
     */
    public static String parseString(Object obj, String replace) {
        return (obj == null) ? replace : obj.toString();
    }

    /**
     * @param strObj
     * @return String[]
     * @Title: split
     * @Description: 将字符串以逗号分拆为字符串数组
     */
    public static String[] split(String strObj) {
        return split(strObj, ",");
    }

    /**
     * @param strObj
     * @param string
     * @return String[]
     * @Title: split
     * @Description: 将字符串按指定符号分拆为字符串数组
     */
    public static String[] split(String strObj, String string) {
        if (strObj == null || "".equals(strObj)) {
            return null;
        }
        return strObj.split(string);
    }

    /**
     * @param arrObj
     * @Title: join
     * @Description: 字符串数据连接
     */
    public static String join(String[] arrObj) {
        return join(arrObj, ",");
    }

    /**
     * @param arrObj
     * @return String
     * @Title: convertString
     * @Description: 字符串数据连接
     */
    public static String join(String[] arrObj, String strJoin) {
        if (arrObj == null || arrObj.length <= 0) {
            return null;
        }
        StringBuilder strRet = new StringBuilder();
        for (int i = 0; i < arrObj.length; i++) {
            strRet.append(arrObj[i]);
            if (i != arrObj.length - 1) {
                strRet.append(strJoin);
            }
        }
        return strRet.toString();
    }

    /**
     * @param bytes
     * @return byte[]
     * @Title: encryptUseMD5
     * @Description: MD5加密
     */
    public static byte[] encryptUseMd5(byte[] bytes) {
        try {
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(bytes);
            return md5.digest();
        } catch (NoSuchAlgorithmException e) {
            return null;
        }
    }

    /**
     * @param bytes
     * @return byte[]
     * @Title: encryptUseSHA
     * @Description: SHA加密
     */
    public static byte[] encryptUseSha(byte[] bytes) {
        try {
            MessageDigest sha = MessageDigest.getInstance("SHA");
            sha.update(bytes);
            return sha.digest();
        } catch (NoSuchAlgorithmException e) {
            return null;
        }
    }

    /**
     * @param publicKey
     * @param bytes
     * @return byte[]
     * @Title: encryptUseRSA
     * @Description: RSA加密
     */
    public static byte[] encryptUseRsa(RSAPublicKey publicKey, byte[] bytes) {
        if (publicKey != null) {
            try {
                Cipher cipher = Cipher.getInstance("RSA");
                cipher.init(Cipher.ENCRYPT_MODE, publicKey);
                return cipher.doFinal(bytes);
            } catch (Exception e) {
                return null;
            }
        }
        return null;
    }

    /**
     * @param privateKey
     * @param bytes
     * @return byte[]
     * @Title: decryptRSA
     * @Description: RSA解密
     */
    public static byte[] decryptUseRsa(RSAPrivateKey privateKey, byte[] bytes) {
        if (privateKey != null) {
            try {
                Cipher cipher = Cipher.getInstance("RSA");
                cipher.init(Cipher.DECRYPT_MODE, privateKey);
                return cipher.doFinal(bytes);
            } catch (Exception e) {
                return null;
            }
        }
        return null;
    }

    public static final InputStream byte2InputStream(byte[] buf) {
        return new ByteArrayInputStream(buf);
    }

    public static final byte[] inputStream2Byte(InputStream inStream)
            throws IOException {
        ByteArrayOutputStream swapStream = new ByteArrayOutputStream();
        byte[] buff = new byte[100];
        int rc = 0;
        while ((rc = inStream.read(buff, 0, 100)) > 0) {
            swapStream.write(buff, 0, rc);
        }
        byte[] in2b = swapStream.toByteArray();
        return in2b;
    }

    /**
     * @param obj
     * @return String
     * @Title: convertArray
     * @Description: 多维数组递归转化为字符串
     */
    public static String convertArray(Object obj) {
        if (obj == null) {
            return "";
        }
        StringBuilder retBuilder = new StringBuilder();
        recurse(obj, retBuilder);
        return retBuilder.toString();
    }

    /**
     * @param obj
     * @param retBuilder void
     * @Title: recurse
     * @Description: 数组递归转化为字符串方法体
     */
    private static void recurse(Object obj, StringBuilder retBuilder) {
        if (obj != null && obj.getClass().isArray()) {
            if (retBuilder.toString().endsWith("]")) {
                retBuilder.append(",[");
            } else {
                retBuilder.append("[");
            }
            for (int i = 0; i < Array.getLength(obj); i++) {
                recurse(Array.get(obj, i), retBuilder);
            }
            retBuilder.append("]");
        } else {
            if (!retBuilder.toString().endsWith("[")) {
                retBuilder.append(",");
            }
            if (obj instanceof String) {
                retBuilder.append("'" + obj + "'");
            } else {
                retBuilder.append(obj);
            }
        }
    }

    /**
     * @param obj
     * @param length
     * @return Object
     * @Title: arrayReduceLength
     * @Description: 对传入数据按指定长度生成新数组
     */
    public static Object arrayReduceLength(Object obj, int length) {
        if (!obj.getClass().isArray()) {
            return null;
        }
        Class componentType = obj.getClass().getComponentType();
        Object newArray = Array.newInstance(componentType, length);
        System.arraycopy(obj, 0, newArray, 0, length);
        return newArray;
    }

    /**
     * @return String
     * @Title: subString
     * @Description: 根据指定开始位置和结束位置截取字符串
     */
    public static String substring(String str, int sPosition, int ePosition) {
        if (str == null) {
            return "";
        }

        String ret;
        if (str.length() <= sPosition) {
            ret = "";
        } else {
            if (str.length() <= ePosition) {
                ret = str.substring(sPosition, str.length());
            } else {
                ret = str.substring(sPosition, ePosition);
            }
        }
        return ret;
    }

    /**
     * @param ordinStr 源字符串
     * @param lenByte  字节长度
     * @return String
     * @Title: substrb
     * @Description: 按字节截取
     */
    public static String substrb(String ordinStr, int lenByte) {
        return substrb(ordinStr, lenByte, null);
    }

    /**
     * @param ordinStr 源字符串
     * @param lenByte  字节长度
     * @param charset  编码方式
     * @return String
     * @Title: substrb
     * @Description: 按字节截取
     */
    public static String substrb(String ordinStr, int lenByte, String charset) {
        if (ValidateUtils.isEmpty(ordinStr)) {
            return "";
        }
        try {
            charset = charset == null ? "UTF-8" : charset;
            if (ordinStr.getBytes(charset).length <= lenByte) {
                return ordinStr;
            }
            String tmp = "";
            for (int i = ordinStr.length() - 1; i >= 0; i--) {
                tmp = ordinStr.substring(0, i);
                if (tmp.getBytes(charset).length <= lenByte) {
                    break;
                }
            }
            // 超过字符长度使用...表示结尾
            if (ordinStr.length() > lenByte) {
                return tmp.substring(0, tmp.length() - 4) + "...";
            } else {
                return tmp;
            }
        } catch (UnsupportedEncodingException e) {
            return ordinStr;
        }
    }

    /**
     * @param str 参数
     * @return String
     * @Title: convertEmpty2Null
     * @Description: 空串转null
     */
    public static String convertEmpty2Null(String str) {
        if (ValidateUtils.isEmpty(str)) {
            return null;
        } else {
            return str;
        }
    }

    /**
     * 解析数据库连接字符串(jdbc:oracle:thin:@192.168.0.200:1522:reliabeach)
     *
     * @return HOST DB PORT
     */
    public static String[] parseUrl(String url) {
        String[] dbParam = new String[3];
        Pattern p = compile("jdbc:oracle:thin:@(.+?):(\\d+):(.*)");
        Matcher m = p.matcher(url);
        while (m.find()) {
            dbParam[0] = m.group(1);
            dbParam[1] = m.group(2);
            dbParam[2] = m.group(3);
        }
        return dbParam;
    }

    /**
     * @param str
     * @return
     */
    public static String getMapValue(String str, Map<String, String> paramMap) {
        if (paramMap.containsKey(str)) {
            return paramMap.get(str);
        } else {
            return "";
        }
    }

    /**
     * @param str
     * @return
     */
    public static String replaceEnterLine(String str) {
        if (ValidateUtils.isEmpty(str)) {
            return "";
        } else {
            return str.replaceAll("\r", "\\\\r")
                    .replaceAll("\n", "\\\\n")
                    .replaceAll("\t", "\\\\t")
                    .replaceAll("\"", "“")
                    .replaceAll("'", "‘");
        }
    }

    /**
     * @param str
     * @param replace
     * @return
     */
    public static String parseVal(String str, String replace) {
        return ValidateUtils.isEmpty(str) ? replace : str;
    }

    /**
     * 数组格式字符串转 String 数组
     *
     * @param source
     * @return
     */
    public static String[] toStringArray(String source) {
        System.out.println(source);
        return source.substring(1, source.length() - 1).split(",");
    }
}
