package com.nationrel.fta.util;

import com.nationrel.modules.fta.general.util.common.SpringContextUtil;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.function.BiConsumer;

/**
 * Mybatis 批量新增工具类
 */
public class BatchInsertUtil {

    private static final Logger logger = LoggerFactory.getLogger(BatchInsertUtil.class);

    /**
     * 批量新增方法
     *
     * @param list 要新增的集合
     * @param clazz Mapper类
     * @param biConsumer 对应的单条新增方法
     * @param <M> mapper类型
     * @param <T> 结合元素类型
     */
    public static <M, T> void batchInsert(List<T> list, Class<M> clazz, BiConsumer<M, T> biConsumer) {
        if (list == null || list.size() == 0) {
            logger.info("BatchInsertUtil batchInsert list data is null!");
            return;
        }
        SqlSessionFactory sqlSessionFactory = SpringContextUtil.getBean(SqlSessionFactory.class);
        SqlSession sqlSession = sqlSessionFactory.openSession(ExecutorType.BATCH);
        try {
            M mapper = sqlSession.getMapper(clazz);
            long start = System.currentTimeMillis();
            for (T t : list) {
                biConsumer.accept(mapper, t);
            }
            logger.info(String.format("批量插入个数：%s,耗时：%s(ms)", list.size(), (System.currentTimeMillis() - start)));
            sqlSession.flushStatements();
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("BatchInsertUtil batchInsert is exception！clazz={}", clazz.getName(), e);
            sqlSession.rollback();
        } finally {
            sqlSession.close();
        }
    }
}