package com.nationrel.modules.fta.util;

import com.nationrel.modules.fta.constant.FtaConstants;
import com.nationrel.modules.fta.entity.FtaAnalysis;
import com.nationrel.modules.fta.entity.FtaEdge;
import com.nationrel.modules.fta.entity.FtaNode;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaNodeEnum;
import com.nationrel.modules.fta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.fta.general.exception.DescribeException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.sql.*;
import java.util.Date;
import java.util.List;

@Component
public class JDBC2Util {

    public static void batchAnalysisInsert(List<FtaAnalysis> analysisList, String mysqlUrl, String mysqlUserName, String mysqlPassword, Integer languageKey) {
        //1.导入驱动jar包
        //2.注册驱动(mysql5之后的驱动jar包可以省略注册驱动的步骤)
        //Class.forName("com.mysql.jdbc.Driver");
        //3.获取数据库连接对象
        Connection conn = null;
        PreparedStatement pstmt = null;
        try {
            //"&rewriteBatchedStatements=true",一次插入多条数据，只插入一次
            conn = DriverManager.getConnection(mysqlUrl.split(" \\?")[0] + "&rewriteBatchedStatements=true", mysqlUserName, mysqlPassword);
            conn.setAutoCommit(false);
            //4.定义sql语句
            String sql = "insert into fta_analysis (analysis_id,analysis_name,project_id,insert_time,insert_user_id,father_id,level) values(?,?,?,now(),?,?,?)";
            //5.获取执行sql的对象PreparedStatement
            pstmt = conn.prepareStatement(sql);
            int resultNum = 0;
            //6.不断产生sql
            for (int i = 0, size = analysisList.size(); i < size; i++) {
                FtaAnalysis analysis = analysisList.get(i);
                if (i != 0 && i % FtaConstants.MAX_BATCH_NUMMAX == 0) {
                    pstmt.executeBatch();
                    pstmt.clearBatch();
                    resultNum = 0;
                }
                resultNum++;
                pstmt.setString(1, analysis.getAnalysisId());
                pstmt.setString(2, analysis.getAnalysisName());
                pstmt.setString(3, analysis.getProjectId());
                pstmt.setString(4, analysis.getCreateBy());
                pstmt.setString(5, analysis.getFatherId());
                pstmt.setInt(6, analysis.getLevel());
                pstmt.addBatch();
            }
            //7.往数据库插入一次数据
            if (resultNum > 0) {
                pstmt.executeBatch();
            }
            conn.commit();//执行完后，手动提交事务
            conn.setAutoCommit(true);//再把自动提交打开，避免影响其他需要自动提交的操作
        } catch (SQLException e) {
            e.printStackTrace();
            throw new DescribeException(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
        } finally {
            //8.释放资源
            //避免空指针异常
            if (pstmt != null) {
                try {
                    pstmt.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public static void batchNodeInsert(List<FtaNode> nodeList, String mysqlUrl, String mysqlUserName, String mysqlPassword, Integer languageKey) {
        //1.导入驱动jar包
        //2.注册驱动(mysql5之后的驱动jar包可以省略注册驱动的步骤)
        //Class.forName("com.mysql.jdbc.Driver");
        //3.获取数据库连接对象
        Connection conn = null;
        PreparedStatement pstmt = null;
        try {
            //"&rewriteBatchedStatements=true",一次插入多条数据，只插入一次
            conn = DriverManager.getConnection(mysqlUrl.split(" \\?")[0] + "&rewriteBatchedStatements=true", mysqlUserName, mysqlPassword);
            conn.setAutoCommit(false);
            //4.定义sql语句
            String sql = "insert into fta_node (ANALYSIS_ID,ID,FATHER_ID,CHILDREN_ANALYSIS_ID,CODE,NAME,TYPE_CODE,DISTRIBUTION_TYPE_CODE,P1_VALUE,P2_VALUE,P3_VALUE,MASTER_ID,REQUIRED_PATH,LEAF,DISABLE_MOVE,NODE_LEVEL) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
            //5.获取执行sql的对象PreparedStatement
            pstmt = conn.prepareStatement(sql);
            int resultNum = 0;
            String number = "%04d";
            int nodeCode = 1;
            if (String.valueOf(nodeList.size()).length() > 4) {
                number = "%0" + String.valueOf(nodeList.size()).length() + "d";
            }
            //6.不断产生sql
            for (int i = 0, size = nodeList.size(); i < size; i++) {
                FtaNode node = nodeList.get(i);
                if (i != 0 && i % FtaConstants.MAX_BATCH_NUMMAX == 0) {
                    pstmt.executeBatch();
                    pstmt.clearBatch();
                    resultNum = 0;
                }
                resultNum++;
                if (StringUtils.isNotBlank(node.getTypeCode()) && FtaNodeEnum.中间事件.getIndex().equals(node.getTypeCode())) {
                    nodeCode++;
                }
                pstmt.setString(1, node.getAnalysisId());
                pstmt.setString(2, node.getId());
                pstmt.setString(3, node.getFatherId());
                if (node.getChildrenAnalysisId() == null) {
                    pstmt.setNull(4, Types.NULL);
                } else {
                    pstmt.setString(4, node.getChildrenAnalysisId());
                }
                if (node.getCode() == null) {
//                    pstmt.setNull(5, Types.NULL);
                    pstmt.setString(5, FtaConstants.EVENT_CODE + String.format(number, nodeCode));
                } else {
                    pstmt.setString(5, node.getCode());
                }
                if (node.getName() == null) {
                    pstmt.setNull(6, Types.NULL);
                } else {
                    pstmt.setString(6, node.getName());
                }
                pstmt.setString(7, node.getTypeCode());
                if (node.getDistributionTypeCode() == null) {
                    pstmt.setNull(8, Types.NULL);
                } else {
                    pstmt.setString(8, node.getDistributionTypeCode());
                }
                if (node.getP1Value() == null) {
                    pstmt.setNull(9, Types.NULL);
                } else {
                    pstmt.setDouble(9, node.getP1Value());
                }
                if (node.getP2Value() == null) {
                    pstmt.setNull(10, Types.NULL);
                } else {
                    pstmt.setDouble(10, node.getP2Value());
                }
                if (node.getP3Value() == null) {
                    pstmt.setNull(11, Types.NULL);
                } else {
                    pstmt.setDouble(11, node.getP3Value());
                }
                if (node.getMasterId() == null) {
                    pstmt.setNull(12, Types.NULL);
                } else {
                    pstmt.setString(12, node.getMasterId());
                }
                if (node.getRequiredPath() == null) {
                    pstmt.setNull(13, Types.NULL);
                } else {
                    pstmt.setInt(13, node.getRequiredPath());
                }
                if (node.getLeaf() == null) {
                    pstmt.setNull(14, Types.NULL);
                } else {
                    pstmt.setBoolean(14, node.getLeaf());
                }
                if (node.getDisableMove() == null) {
                    pstmt.setNull(15, Types.NULL);
                } else {
                    pstmt.setBoolean(15, node.getDisableMove());
                }
                if (node.getNodeLevel() == null) {
                    pstmt.setNull(16, Types.NULL);
                } else {
                    pstmt.setInt(16, node.getNodeLevel());
                }
                pstmt.addBatch();
            }
            //7.往数据库插入一次数据
            if (resultNum > 0) {
                pstmt.executeBatch();
            }
            conn.commit();//执行完后，手动提交事务
            conn.setAutoCommit(true);//再把自动提交打开，避免影响其他需要自动提交的操作
        } catch (SQLException e) {
            e.printStackTrace();
            throw new DescribeException(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
        } finally {
            //8.释放资源
            //避免空指针异常
            if (pstmt != null) {
                try {
                    pstmt.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                    throw new DescribeException(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
                }
            }
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                    throw new DescribeException(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
                }
            }
        }
    }

    public static void batchEdgeInsert(List<FtaEdge> edgeList, String mysqlUrl, String mysqlUserName, String mysqlPassword, Integer languageKey) {
        //1.导入驱动jar包
        //2.注册驱动(mysql5之后的驱动jar包可以省略注册驱动的步骤)
        //Class.forName("com.mysql.jdbc.Driver");
        //3.获取数据库连接对象
        Connection conn = null;
        PreparedStatement pstmt = null;
        try {
            //"&rewriteBatchedStatements=true",一次插入多条数据，只插入一次
            conn = DriverManager.getConnection(mysqlUrl.split(" \\?")[0] + "&rewriteBatchedStatements=true", mysqlUserName, mysqlPassword);
            conn.setAutoCommit(false);
            //4.定义sql语句
            String sql = "insert into fta_edge (ANALYSIS_ID,ID,SOURCE_ID,TARGET_ID,EDGE_ORDER) values(?,?,?,?,?)";
            //5.获取执行sql的对象PreparedStatement
            pstmt = conn.prepareStatement(sql);
            int resultNum = 0;
            //6.不断产生sql
            for (int i = 0, size = edgeList.size(); i < size; i++) {
                FtaEdge edge = edgeList.get(i);
                if (i != 0 && i % FtaConstants.MAX_BATCH_NUMMAX == 0) {
                    pstmt.executeBatch();
                    pstmt.clearBatch();
                    resultNum = 0;
                }
                resultNum++;
                pstmt.setString(1, edge.getAnalysisId());
                pstmt.setString(2, edge.getId());
                pstmt.setString(3, edge.getSourceId());
                pstmt.setString(4, edge.getTargetId());
                if (edge.getEdgeOrder() == null) {
                    pstmt.setNull(5, Types.NULL);
                } else {
                    pstmt.setInt(5, edge.getEdgeOrder());
                }
                pstmt.addBatch();
            }
            //7.往数据库插入一次数据
            if (resultNum > 0) {
                pstmt.executeBatch();
            }
            conn.commit();//执行完后，手动提交事务
            conn.setAutoCommit(true);//再把自动提交打开，避免影响其他需要自动提交的操作
        } catch (SQLException e) {
            e.printStackTrace();
            throw new DescribeException(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
        } finally {
            //8.释放资源
            //避免空指针异常
            if (pstmt != null) {
                try {
                    pstmt.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                    throw new DescribeException(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
                }
            }
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                    throw new DescribeException(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
                }
            }
        }
    }

    public static Timestamp getTimestamp(Date date) {
        if (date != null) {
            return new Timestamp(date.getTime());
        }
        return null;
    }

}
