package com.nationrel.modules.fta.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.nationrel.modules.fta.entity.FtaProject;
import com.nationrel.modules.fta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.fta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.fta.handler.GlobalExceptionHandler;
import com.nationrel.modules.fta.service.FtaProjectService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

@Tag(name = "FTA-项目")
@RestController
@RequestMapping("/fta/project")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FtaProjectController {

    private final FtaProjectService analysisService;

    @Operation(summary = "项目列表")
    @GetMapping("/list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "altaId", description = "altaId", in = ParameterIn.QUERY),
            @Parameter(name = "projectName", description = "分析名称", in = ParameterIn.QUERY),
            @Parameter(name = "startDate", description = "开始时间", in = ParameterIn.QUERY),
            @Parameter(name = "endDate", description = "结束时间", in = ParameterIn.QUERY),
            @Parameter(name = "pageIndex", description = "当前页码", in = ParameterIn.QUERY),
            @Parameter(name = "pageSize", description = "每页条数", in = ParameterIn.QUERY)
    })
    public Result<IPage<FtaProject>> analysisList(
            @RequestParam(required = false) String projectId,
            @RequestParam(required = false) String projectName,
            @RequestParam(required = false) Date startDate,
            @RequestParam(required = false) Date endDate,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return analysisService.listByPage(userInfo, projectId, projectName, startDate, endDate, pageIndex, pageSize);
    }

    @Operation(summary = "项目创建")
    @PostMapping("/add")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public Result<String> createAnalysis(HttpServletRequest request, @RequestBody FtaProject entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(entity.getProjectName())) {
            return Result.error(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.分析名称不可以为空, userInfo.getLanguageKey()));
        }
        return analysisService.addAnalysis(userInfo, entity);
    }

    @Operation(summary = "项目修改")
    @PostMapping("/edit")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public Result<String> updateAnalysis(@RequestBody FtaProject entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(entity.getProjectName())) {
            return Result.error(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.分析名称不可以为空, userInfo.getLanguageKey()));
        }
        return analysisService.updateAnalysisInfo(userInfo, entity);
    }

    @Operation(summary = "项目删除")
    @DeleteMapping("/delete")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public Result<String> deleteAnalysis(@RequestParam(name = "ftaId", required = true) String ftaId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(ftaId)) {
            return Result.error(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.分析名称不可以为空, userInfo.getLanguageKey()));
        }
        return analysisService.deleteAnalysisInfo(userInfo, ftaId);
    }

}
