package com.nationrel.modules.fta.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.nationrel.modules.fta.entity.FtaEdge;
import com.nationrel.modules.fta.entity.FtaNode;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;

public interface FtaNodeMapper extends BaseMapper<FtaNode> {

    /**
     * 批量更新
     *
     * @param list
     */
    void updateNodes(@Param("list") List<FtaNode> list);

    /**
     * 批量更新重复事件
     *
     * @param list
     */
    void updateMirrorNodes(@Param("list") List<FtaNode> list);

    /**
     * 批量更新重复事件的父节点的编号和名称
     *
     * @param list
     */
    void updateMirrorFatherNodes(@Param("list") List<FtaNode> list);

    /**
     * 批量修噶节点的类型
     *
     * @param list
     */
    void updateTypeCodes(@Param("list") List<FtaNode> list, @Param("typeCode") String typeCode);

    /**
     * 批量更新计算结果：Q
     *
     * @param list
     */
    void updateNodesCalData(@Param("list") List<FtaNode> list);

    /**
     * 批量更新新的分析Id（更新为转移节点的分析Id）
     *
     * @param ids
     * @param newAnalysisId
     */
    void updateNodesAnalysisId(@Param("list") List<String> ids, @Param("analysisId") String newAnalysisId);

    /**
     * 清除计算历史
     *
     * @param ftaIds
     */
    void clearCalHistoryByIds(@Param("list") List<String> ftaIds);

    /**
     * 清除计算历史
     *
     * @param nodeIds
     */
    void clearCalHistoryByNodeIds(@Param("list") List<String> nodeIds);

    /**
     * 清除计算历史
     *
     * @param ftaId
     */
    void clearCalHistoryById(@Param("id") String ftaId);

    List<FtaNode> selectLastLevel(@Param("analysisId") String analysisId, @Param("nodeLevel") int nodeLevel);

    @Select("SELECT MAX(node_level) FROM fta_node WHERE analysis_id = #{analysisId} and del_flag = '0'")
    int selectMaxnodeLevel(@Param("analysisId") String analysisId);

    @Select("SELECT id FROM fta_node WHERE analysis_id = #{analysisId} and node_level = 1 and father_id != '0' and del_flag = '0'")
    String selectFirstLevelNodeId(@Param("analysisId") String analysisId);

    @Select("SELECT MAX(edge_order) FROM fta_node WHERE analysis_id = #{analysisId} and father_id = #{sourceId} and del_flag = '0'")
    int selectMaxTwoLevelOrder(@Param("analysisId") String analysisId, @Param("sourceId") String sourceId);

    /**
     * 查询该节点id是否存在存活的关联的masterId
     *
     * @param nodeId
     * @return
     */
    List<String> selectNodeMasterIds(@Param("nodeId") String nodeId);

    /**
     * 根据analysisId查询节点绘图相关的基本信息及父节点的code
     *
     * @param analysisId
     * @return
     */
    @Select("SELECT a.id id, a.master_id masterId, a.type_code typeCode, a.distribution_type_code distributionTypeCode, a.p1_value p1Value, a.p2_value p2Value, a.p3_value p3Value, b.code code FROM fta_node a LEFT JOIN fta_node b ON a.father_id = b.id WHERE a.analysis_id = #{analysisId} and a.type_code IN (3,4,5) and a.del_flag = '0'")
    List<FtaNode> selectIdAndFatherName(@Param("analysisId") String analysisId);

    /**
     * 批量更新顺序
     *
     * @param list
     */
    void updateEdgeOrder(@Param("list") List<FtaEdge> list);

    /**
     * 更新连线顺序（左移），将sourceId下的连线中的 sourceOrder向左移动到targetOrder
     *
     * @param sourceId    这些连线的sourceId
     * @param sourceOrder 需要移动的连线
     * @param targetOrder 需要移动到的目标位置
     */
    @Update("UPDATE fta_node SET edge_order = edge_order + 1 WHERE analysis_id = #{analysisId} AND father_id = #{sourceId} AND edge_order >= #{targetOrder} AND edge_order < #{sourceOrder} AND del_flag = '0'")
    void moveToRight(@Param("analysisId") String analysisId, @Param("sourceId") String sourceId, @Param("sourceOrder") Integer sourceOrder, @Param("targetOrder") Integer targetOrder);

    /**
     * 更新连线顺序（右移），将sourceId下的连线中的 sourceOrder移动到targetOrder
     *
     * @param sourceId    这些连线的sourceId
     * @param sourceOrder 需要移动的连线
     * @param targetOrder 需要移动到的目标位置
     */
    @Update("UPDATE fta_node SET edge_order = edge_order - 1 WHERE analysis_id = #{analysisId} AND father_id = #{sourceId} AND edge_order > #{sourceOrder} AND edge_order <= #{targetOrder} AND del_flag = '0'")
    void moveToLeft(@Param("analysisId") String analysisId, @Param("sourceId") String sourceId, @Param("sourceOrder") Integer sourceOrder, @Param("targetOrder") Integer targetOrder);

    /**
     * 更新连线顺序,将sourceId下 大于等于指定顺序的连线的顺序+1（排除指定连线notInIds）
     *
     * @param sourceId
     * @param notInIds 需要排除的节点id
     * @param order
     */
    @Update("UPDATE fta_node SET edge_order = edge_order + 1 WHERE father_id = #{sourceId} AND edge_order >= #{order} AND id <> #{notInIds} AND del_flag = '0'")
    void moveRightNotIn(@Param("sourceId") String sourceId, @Param("notInIds") String notInIds, @Param("order") Integer order);

    /**
     * 更新连线顺序,将sourceId下 大于指定顺序的连线的顺序-1（排除指定连线notInIds）
     *
     * @param sourceId
     * @param notInIds
     * @param order
     */
    @Update("UPDATE fta_node SET edge_order = edge_order - 1 WHERE father_id = #{sourceId} AND edge_order > #{order} AND id <> #{notInIds} AND del_flag = '0'")
    void moveLeftNotIn(@Param("sourceId") String sourceId, @Param("notInIds") String notInIds, @Param("order") Integer order);

    /**
     * 更新连线顺序,将sourceId下 大于指定顺序的连线的顺序-1
     *
     * @param sourceId
     * @param order
     */
    @Update("UPDATE fta_node SET edge_order = edge_order - 1 WHERE analysis_id = #{analysisId} AND father_id = #{sourceId} AND edge_order > #{order} AND del_flag = '0'")
    void moveLeft(@Param("sourceId") String sourceId, @Param("analysisId") String analysisId, @Param("order") Integer order);

    /**
     * 更新连线顺序,将sourceId下 大于等于指定顺序的连线的顺序+1
     *
     * @param sourceId
     * @param order
     */
    @Update("UPDATE fta_node SET edge_order = edge_order + 1 WHERE analysis_id = #{analysisId} AND father_id = #{sourceId} AND edge_order >= #{order} AND del_flag = '0'")
    void moveRight(@Param("sourceId") String sourceId, @Param("analysisId") String analysisId, @Param("order") Integer order);

    /**
     * 更新连线顺序,将sourceId下 大于指定顺序的连线的顺序-1（排除指定连线targetId）
     *
     * @param sourceId 连线的sourceId
     * @param order    需要参考的顺序
     */
    void moveLeftNotIns(@Param("sourceId") String sourceId, @Param("list") List<String> targetId, @Param("order") Integer order);

    @Select("select id from fta_node where children_analysis_id = #{analysisId}")
    String selectByChildrenAnalysisId(@Param("analysisId") String analysisId);
}




