// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.fta.service;

import com.alibaba.fastjson.JSONObject;
import com.nationrel.modules.fta.config.AllVariableConfig;
import com.nationrel.modules.fta.constant.FtaConstants;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaDistributingTypeCodeEnum;
import com.nationrel.modules.fta.general.util.common.ValidateUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @version V1.0
 * @ClassName RldAnalysisChartDataServiceImpl
 * @Description api调用
 * @Author aaron
 * @CreateDate 2018-11-1
 */
@Slf4j
@Service
@RefreshScope
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FtaRgaApiService {

    private final AllVariableConfig allVariableConfig;

    private int retry;

    public String getResultsCallApi(String requestData, String url, String contentType, String acceptType) {
        PostMethod postMethod = null;
        String responseResult = "";
        try {
            HttpClient clientPlot = new HttpClient();
            postMethod = new PostMethod(url);
            System.err.println(requestData);
            System.err.println(url);
            RequestEntity requestEntityPlot = new StringRequestEntity(requestData, "text/xml", "UTF-8");
            postMethod.setRequestEntity(requestEntityPlot);
            postMethod.setRequestHeader("Content-Type", contentType);
            if (ValidateUtils.isNotEmpty(acceptType)) {
                postMethod.setRequestHeader("Accept", acceptType);
            }
            clientPlot.executeMethod(postMethod);
            // responseResult = new
            // String(postMethod.getResponseBodyAsString().getBytes("UTF-8"));
            InputStream inputStream = postMethod.getResponseBodyAsStream();
            BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
            StringBuilder stringBuffer = new StringBuilder();
            String str;
            while ((str = br.readLine()) != null) {
                stringBuffer.append(str);
            }
            responseResult = stringBuffer.toString();
        } catch (Exception e) {
            if (retry < 3) {
                retry++;
                // 如果连接失效，重试
                getResultsCallApi(requestData, url, contentType, acceptType);
            } else {
                retry = 0;
                e.printStackTrace();
//                throw new Exception("无法链接服务器");
            }
        } finally {
            Objects.requireNonNull(postMethod).releaseConnection();
        }
        return responseResult;
    }

    public String getResultsCallApi(JSONObject requestData, String url, String contentType, String acceptType) {
//        log.error(requestData.toString());
        long l = System.currentTimeMillis();
        PostMethod postMethod = null;
        String responseResult = "";
        try {
            HttpClient clientPlot = new HttpClient();
            postMethod = new PostMethod(url);
            RequestEntity requestEntityPlot = new StringRequestEntity(
                    requestData.toString(), "text/xml", "UTF-8");
            postMethod.setRequestEntity(requestEntityPlot);
            postMethod.setRequestHeader("Content-Type", contentType);
            if (StringUtils.isNotEmpty(acceptType)) {
                postMethod.setRequestHeader("Accept", acceptType);
            }
            clientPlot.executeMethod(postMethod);
            responseResult = new String(postMethod.getResponseBodyAsString().getBytes(StandardCharsets.UTF_8));
        } catch (Exception e) {
            if (retry < 3) {
                retry++;
                // 如果连接失效，重试
                getResultsCallApi(requestData, url, contentType, acceptType);
            } else {
                retry = 0;
                e.printStackTrace();
                //throw new ApplicationRuntimeException("无法链接寿命数据算法服务器");
            }
        } finally {
            postMethod.releaseConnection();
        }
        log.error("接口用时：" +String.valueOf(System.currentTimeMillis() - l));
        return responseResult;
    }

    /**
     * 获取系统方程uri
     *
     * @return
     */
    public String getFtaGenerateSysEquationUri() {
        return allVariableConfig.getReliaQubeFtaUrl() + FtaConstants.FTA_GENERATE_SYS_CUTSETS;
    }

    /**
     * 绘图uri（计算器）
     *
     * @return
     */
    public String getFtaCalSysReliabilityMatricUri() {
        return allVariableConfig.getReliaQubeFtaUrl() + FtaConstants.FTA_CAL_SYS_RELIABILITY_MATRIC;
    }

    /**
     * 节点绘图
     */
    public String getApiPlotUrl(String distributionTypeCode, String plotType, String moreParameter) {

        // 接口调用路径
        String apiPlotUrl = allVariableConfig.getReliaQubeUrl() + FtaConstants.API_PLOT_URL;
        if (FtaConstants.PLOT_TYPE_RELIABILITY.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "ReliabilityPlotData";
        } else if (FtaConstants.PLOT_TYPE_UNRELIABILITY.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "UnReliabilityPlotData";
        } else if (FtaConstants.PLOT_TYPE_FAILURERATE.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "FailureRatePlotData";
        } else if (FtaConstants.PLOT_TYPE_PDF.equals(plotType)) {
            apiPlotUrl = apiPlotUrl + "PdfPlotData";
        }

        if (FtaDistributingTypeCodeEnum.Weibull2.getIndex().equals(distributionTypeCode)) {
            // apiPlotUrl = apiPlotUrl+"?"+"distribution=Weibull";
            apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Weibull";
        } else if (FtaDistributingTypeCodeEnum.Weibull3.getIndex().equals(distributionTypeCode)) {
            // apiPlotUrl = apiPlotUrl+"?"+"distribution=Weibull3P";
            apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Weibull3P";
        } else if (FtaDistributingTypeCodeEnum.Lognormal.getIndex().equals(distributionTypeCode)) {
            // apiPlotUrl = apiPlotUrl+"?"+"distribution=Lognormal";
            apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Lognormal";
        } else if (FtaDistributingTypeCodeEnum.Normal.getIndex().equals(distributionTypeCode)) {
            // apiPlotUrl = apiPlotUrl+"?"+"distribution=Normal";
            apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Normal";
        } else if (FtaDistributingTypeCodeEnum.Exponential.getIndex().equals(distributionTypeCode)) {
            // apiPlotUrl = apiPlotUrl+"?"+"distribution=Exponential";
            apiPlotUrl = apiPlotUrl + moreParameter + "&distribution=Exponential";
        }
        return apiPlotUrl;
    }
}
